package algorithms;

import graph.AdjacencyMatrixGraph;
import graph.ListColoredAdjacencyMatrixGraph;
import importer.Results.Instance;
import java.util.*;
//import algorithms.Solver.MatchingType;

public class MaxIterativeSolver extends Solver {

	private MatchingType type;
	private final boolean verbose = false;
	private Instance instance;
	private int n;
	
	public MaxIterativeSolver(ListColoredAdjacencyMatrixGraph g, AdjacencyMatrixGraph h, MatchingType m, Instance instance){
		super(g,h);
		if (m == MatchingType.MaxHomomorph) type = MatchingType.Homomorph;
		else type = MatchingType.Isomorph;
		this.instance = instance;
		n = g.vertexNum();
	}
	
	public boolean solve() {
		//create a copy of the input graph
		boolean solved = (new IterativeSolver(g,h,type,instance)).solve();
		if (solved) safeprint("Found in the first try");
		int[] del =  null;
		while (!solved){
			safeprint("test");
			//create next deletion set
			del = getNext(del);
			safeprint("del");
			for (int i =0;i<del.length;i++){
				safeprint(del[i]);
			}
			
			//abort if all but one vertex of g are deleted
			if (del.length>=n-1) break;
			
			//create integer set for induced subgraph procedure
			Set<Integer> delSet = new HashSet<Integer>();
			for (int i=0;i<del.length;i++){
				delSet.add(del[i]);
			}
			
			//create new graph g2 and new instance of iterative solver
			ListColoredAdjacencyMatrixGraph g2 = g.inducedGraphVertexDel(delSet);
			//solve instance for these graphs
			IterativeSolver solver = new IterativeSolver(g2,h,type,instance);
			solved = solver.solve();
		}
		return solved;
	}

	private int[] getNext(int[] del){
		//start with deleting vertex '0'
		if (del == null){
			int[] newArray = new int[1];
			return newArray;
		}
		//if last deletion set of size k has been tried, generate first deletion set of size k+1
		else if (del[del.length-1] == (n-(del.length))){
			int[] newArray = new int[del.length+1];
			for (int i=0;i<del.length;i++){
				newArray[i]=del.length-i;
			}
			return newArray;
		}
		else{
			//find first position to increase
			for (int i=0;i<del.length;i++){
				if (del[i]< n-i-1){
					del[i]=del[i]+1;
					break;
				}
				else if (del[i+1]<n-i-2){
					del[i+1]=del[i+1]+1;
					for (int j = i;j>=0;j--){
						del[j] =del [j+1]+1;
					}
					break;
				}
			}
			
			return del;
		}
		
	}

	private void safeprint(Object s){
		if (verbose) System.out.println(s);
	}
}
