package algorithms;
import importer.Results.Instance;
import graph.*;

public abstract class Solver {
	ListColoredAdjacencyMatrixGraph g;
	AdjacencyMatrixGraph h;
	
	public enum MatchingType {Homomorph,Isomorph,MaxHomomorph,MaxIsomorph,All};
	public enum AlgorithmType {SearchTree,IterativeSearchTree};
	public Solver(){}

	/**
	 * Generates an instance of the Bounded-list Morphism Problem from two input graphs.
	 *
	 * @param  g  the pattern
	 * @param  h  the host
	 */
	
	public Solver(ListColoredAdjacencyMatrixGraph g, AdjacencyMatrixGraph h){
		this.g = g; 
		this.h = h;
	}
	
	public abstract boolean solve();

	public static Solver solverFactory(ListColoredAdjacencyMatrixGraph g, AdjacencyMatrixGraph h, MatchingType m, Instance instance, AlgorithmType algoType){
		if (algoType == Solver.AlgorithmType.SearchTree) {
			return new SearchTreeSolver(g,h,m,instance);
		}
		else {
			if ((m == MatchingType.MaxHomomorph) || (m == MatchingType.MaxIsomorph)){
				return new MaxIterativeSolver(g,h,m,instance);
			}
			else return new IterativeSolver(g,h,m,instance);
		}
		
	}
}
