package goAdapter;

import java.io.IOException;
import java.util.Collection;
import java.util.*;
import java.io.*;
import importer.*;

public class GOAdapter {
	String scriptPath, oboPath, assocPath;
	AnnotationCollection annotations;
	int geneNumber;
	public GOAdapter(String s, String a, String o, int n){
		this.scriptPath = s;
		this.assocPath = a;
		this.oboPath = o;
		this.geneNumber = n;
		this.annotations = new AnnotationCollection();
	}

	public void searchTerms(Collection<String> comp){
		OutWriter out = OutWriter.temporaryOut();
		writeCompFile(comp,out);
		try {
			Runtime.getRuntime().exec("perl "+scriptPath+" "+assocPath+" "+geneNumber+" "+oboPath+" "+out.getFileName());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void searchTermsAll(Collection<Collection<String>> comp){
		String namesList = "";
		Set<OutWriter> outSet = new HashSet<OutWriter>();
		for (Collection<String> c: comp){
			OutWriter out = OutWriter.temporaryOut();
			writeCompFile(c,out);
			outSet.add(out);
			//System.out.println(namesList);
			namesList = namesList + out.getFileName()+" ";
		}
		try {
			System.out.println("call analyzer for: "+namesList);
			Process p = Runtime.getRuntime().exec("perl "+scriptPath+" "+assocPath+" "+geneNumber+" "+oboPath+" "+namesList);
			p.waitFor();
			System.out.println("Done...");
		} catch (Exception e) {
			e.printStackTrace();
		}
		for (OutWriter out: outSet){
			annotations.add(readAnnotation(out.getFileName()+".terms"));
		}
		System.out.println(annotations.getEnrichmentStats());
	}
	
	public void writeCompFile(Collection<String> comp, OutWriter out){
		for (String c:comp){
			out.write(c);
		}
		out.close();
	}
	
	public AnnotatedSet readAnnotation(String path){
		AnnotatedSet aSet= new AnnotatedSet();
		try{
			File f = new File(path);
			BufferedReader reader = new BufferedReader(new FileReader(f));
			String line = null;
			Annotation.type aType = Annotation.type.None;
			while ((line = reader.readLine()) != null) {
				//System.out.println(line);
				if (line.startsWith("Finding terms for P")) aType = Annotation.type.Process;
				else if (line.startsWith("Finding terms for C")) aType = Annotation.type.Component;
				else if (line.startsWith("Finding terms for F")) aType = Annotation.type.Function;
				else if (line.startsWith("CORRECTED")){
					String copy = line;
					double p = pVal(new StringTokenizer(copy));
					//System.out.println(p);
					if (p <0.05) {
						//System.out.println("Yesss");
						aSet.setAnnotation(aType); 
					}
				}
			}
		}
		catch (Exception e) {
			System.out.println(e.getMessage());
		}
		return aSet;
	}
	
	public String getAnnotationStats(){
		return this.annotations.getEnrichmentStats();
	}
	
	private double pVal(StringTokenizer tab){
		tab.nextToken();tab.nextToken();
		return Double.valueOf(tab.nextToken());
	}
	
}
