package importer;

import java.util.*;
import java.io.*;

public class Results {
	String path;
	public Set<Instance> instances;
	double overallTime;
	
	public class Instance{
		Set<String> complex,subcomplex,candidates;
		String threshold,algorithm,cPath,hPath,name;
		double runningTime;
		Map<String,String> matches;
		boolean terminated=false;
		
		public Instance(String n, Set<String> complex, Set<String> subcomplex, Set<String> candidates){
			this.name = n;
			this.complex = complex;
			this.subcomplex = subcomplex;
			this.candidates = candidates;
		}
		
		public void addMatching(Map<String,String> matches){
			this.matches = matches;
		}
		
		public void writeMatch(String path){
			if (matches!=null){
				String filename = name.replace('/', '-');
				OutWriter out = new OutWriter(path+filename+".dat");
				for (String prot: matches.values()){
					out.write(prot);
				}
				out.close();
			}
		}
		
		public void addRunningTime(double r){
			this.runningTime = r;
		}
		
		public int size(){
			return subcomplex.size();
		}
		
		public void terminate(){
			this.terminated = true;
		}
		
		public boolean hasMatches(){
			if (matches!=null) return true;
			else return false;
		}
		
	}
	
	
	public Results(String path){
		this.path = path;
		this.instances = new HashSet<Instance>();
		try{
			(new File(path)).mkdirs();
		}
		catch (Exception e) {
			System.out.println("error: "+e.getMessage());
		}
	}
	
	public void addInstance(Instance i){
		instances.add(i);
	}
	
	/*
	 * 
	 */
	public void writeMatches(){
		for (Instance i: this.instances){
			i.writeMatch(path);
		}
	}
	
	public void writeMatches(int minSize){
		for (Instance i: this.instances){
			if (i.size()>=minSize) i.writeMatch(path);
		}
	}
	
	/*
	 * generates running time stats etc. and writes them into a file
	 */
	public void writeStatistics(){
		HashMap<Integer,Double> timeForSize = new HashMap<Integer, Double>();
		HashMap<Integer,Integer> numForSize = new HashMap<Integer, Integer>();
		HashMap<Integer,Integer> unsolvedForSize = new HashMap<Integer, Integer>();		
		for (Instance i: instances){
			if (i.terminated){
				protectedAdd(unsolvedForSize,i.size(),1);
				protectedAdd(numForSize,i.size(),1);				
			}	
			else {
				protectedAdd(timeForSize,i.size(),i.runningTime);
				protectedAdd(unsolvedForSize,i.size(),0);
				protectedAdd(numForSize,i.size(),1);	
			}				
		}
		OutWriter out = new OutWriter(path+"stats.txt");
		out.write("Size"+"\t"+"Number"+"\t"+"Avg. time"+"\t"+"Unfinished");
		for (int i: numForSize.keySet()){			
			double time = Double.MAX_VALUE;
			if (timeForSize.containsKey(i)){
				time = (timeForSize.get(i) / Double.valueOf(numForSize.get(i)-unsolvedForSize.get(i)));
			}
			out.write(i+"\t"
					+numForSize.get(i)+"\t"
					+time+"\t"
					+unsolvedForSize.get(i));
		}
		out.close();
	}
	
	private void protectedAdd(HashMap<Integer,Integer> map, int key, int val){
		if (map.containsKey(key)){
			map.put(key,map.get(key)+val);
		}
		else map.put(key,val);
	}
	
	private void protectedAdd(HashMap<Integer,Double> map, int key, double val){
		if (map.containsKey(key)){
			map.put(key,map.get(key)+val);
		}
		else map.put(key,val);
	}
	
	public void addRunningTime(double r){
		this.overallTime = r;
	}
	
	public Instance addInstance(String name, Set<String> complex, Set<String> subcomplex, Set<String> candidates){
		Instance i= new Instance(name, complex, subcomplex, candidates);
		addInstance(i);
		return i;
	}
	
	public Set<Collection<String>> getComplexes(int minSize){
		Set<Collection<String>> complexSet = new  HashSet<Collection<String>>();
		for (Instance i: instances){
			//System.out.println(i.size());
			if (i.size()>=minSize&&i.hasMatches()) {
				//System.out.println("test");
				complexSet.add(i.matches.values());
			}
		}
		return complexSet;
	}
	
}
